/******************************* Copyright notice *******************************
Copyright (C)2002 by Andrei Loskutov <loskutov@gmx.de>. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
********************************************************************************/

/**
 * Created on 29.11.2002
 *
 * To change this generated comment edit the template variable "filecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of file comments go to
 * Window>Preferences>Java>Code Generation.
 */
package adiag.connection;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Properties;

import adiag.model.Const;

    /**
     * Administrator class for settings and config - file
     * @author Andy the Great
     */
public class PropertySetter extends Observable {
    
    /** is props file read yet? */
    private boolean init;

    /** current application properties */
    private Properties props;
    
    private String changed_property;


    /**
     * Constructor for PropertySetter.
     */
    public PropertySetter() {
        super();
    }

    /**
     * See return description
     * @return Properties curent config object
     */
    public Properties getProps(Configuration conf) {
        if (props == null) {
            try {
                readProps(conf);
            } catch (IOException e) {
                conf.error("Cannot read properties file:\n" + e.getMessage(), e);
            }
        }
        return props;
    }
    
    /**
     * See return description
     * @return ArrayList with all props keys
     */
    public ArrayList getKeyList(Configuration conf){
        return new ArrayList(getProps(conf).keySet());
    }

    public String getLastChangedProperty(){
        return this.changed_property;
    }

    /** 
     * Write props file to HD
     * @throws ExplainedException, if something goes wrong
     */
    public void storeProperties(Configuration conf) throws IOException {
        OutputStream out;
        File f = new File(Configuration.getIniUrl());            
        out = new BufferedOutputStream(new FileOutputStream(f));
        getProps(conf).store(out, "BranchView tool configuration file, please don't edit if you don't understand!");
        out.close();            
    }

    /**
     * See return description
     * @return boolean true, if props file are read
     */
    public boolean isInitDone(){
        return init;
    }

    /**
     * Read props from HD
     * @throws IOException if something goes wrong
     */
    public void readProps(Configuration conf) throws IOException {
        if (props == null) {
            props = new Properties();
        }

        File f = new File(Configuration.getIniUrl());
        InputStream is;
        try {
            is = new BufferedInputStream(new FileInputStream(f));
            props.load(is);
            is.close();                
        } catch (FileNotFoundException e) {
            try {
                f.createNewFile();
            } catch (Exception ex) {
                String message =
                    "The ini - file " + f.getPath()
                        + " cannot be created.\n";                    
                conf.error(message + e.getMessage(), e);
            }
        }
        // set to default
        for (int i = 0; i < Const.PROPERTIES.length; i++) {
            if (!props.containsKey(Const.PROPERTIES[i])) {
                props.put(Const.PROPERTIES[i], conf.getDefaultProperty(Const.PROPERTIES[i]));
            }
        }
        init = true;
    }
         

    /**
     * Set current value for given props key, and update all known windows 
     * instances
     * @param key option name, one of Const.PROPERTY_ - constants
     * @param value current value for given props key
     */
    public void setProperty(String key, String value, Configuration conf){
        key = key == null ? "" : key.trim();
        if(Const.PROPERTY_P4_PASSWD.equals(key)){
            if(value!=null && value.length()!=0 ){
                // encode passwort
                try {                    
                    value = Configuration.toStringBlock(value.getBytes());
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }            
        }
        getProps(conf).setProperty(key, value);
        changed_property = key;
        this.setChanged();
        this.notifyObservers(value);        
        // reset
        changed_property = null;        
    }

}    
    
